@@ -4,7 +4,7 @@ import tty2img
 setuptools.setup(
     name="tty2img",
     py_modules=["tty2img"],
-    version="0.2.1",
+    version="0.3.0",
     author="Robert Paciorek",
     author_email="robert@opcode.eu.org",
     description="rendering pyte terminal emulator screen as image",
@@ -13,7 +13,8 @@ setuptools.setup(
     keywords='asciicast video movie mp4 tty pyte moviepy',
     install_requires=[
         'pyte',
-        'pillow'
+        'pillow',
+        'fclist'
     ],
     license='MIT',
     url="https://bitbucket.org/OpCode-eu-org/asciicast2movie/",
